package cn.com.qmth.examcloud.core.basic.api.controller;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import javax.persistence.criteria.Subquery;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.fileupload.disk.DiskFileItem;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.helpers.poi.ExcelWriter;
import cn.com.qmth.examcloud.commons.util.PathUtil;
import cn.com.qmth.examcloud.core.basic.api.controller.bean.SpecialtyDomain;
import cn.com.qmth.examcloud.core.basic.dao.CourseRepo;
import cn.com.qmth.examcloud.core.basic.dao.CourseSpeciatlyRelationRepo;
import cn.com.qmth.examcloud.core.basic.dao.SpecialtyRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.CourseSpeciatlyRelationEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.SpecialtyEntity;
import cn.com.qmth.examcloud.core.basic.service.SpecialtyService;
import cn.com.qmth.examcloud.core.basic.service.bean.SpecialtyInfo;
import cn.com.qmth.examcloud.web.config.SystemProperties;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.ApiOperation;

/**
 * @Description: 专业服务API
 * @author weiwenhai
 * @Date 2017年8月7号
 */
@RestController
@RequestMapping("${$rmp.ctr.basic}/specialty")
public class SpecialtyController extends ControllerSupport {

	@Autowired
	SpecialtyService specialtyService;

	@Autowired
	SystemProperties systemConfig;

	@Autowired
	SpecialtyRepo specialtyRepo;

	@Autowired
	CourseRepo courseRepo;

	@Autowired
	CourseSpeciatlyRelationRepo courseSpeciatlyRelationRepo;

	private static final String[] EXCEL_HEADER = new String[]{"专业名称", "专业代码"};

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param curPage
	 * @param pageSize
	 * @param name
	 * @param code
	 * @param enable
	 * @param courseId
	 * @return
	 */
	@ApiOperation(value = "分页查询专业")
	@GetMapping("specialtyPage/{curPage}/{pageSize}")
	public Page<SpecialtyEntity> getSpecialtyPage(@PathVariable Integer curPage,
			@PathVariable Integer pageSize, @RequestParam(required = false) String name,
			@RequestParam(required = false) String code,
			@RequestParam(required = false) Boolean enable,
			@RequestParam(required = false) Long courseId) {
		User accessUser = getAccessUser();

		Specification<SpecialtyEntity> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();

			predicates.add(cb.equal(root.get("rootOrgId"), accessUser.getRootOrgId()));

			if (StringUtils.isNotBlank(name)) {
				predicates.add(cb.like(root.get("name"), toSqlSearchPattern(name)));
			}
			if (StringUtils.isNotBlank(code)) {
				predicates.add(cb.like(root.get("code"), toSqlSearchPattern(code)));
			}
			if (null != enable) {
				predicates.add(cb.equal(root.get("enable"), enable));
			}

			if (null != courseId) {
				Subquery<CourseSpeciatlyRelationEntity> subquery = query
						.subquery(CourseSpeciatlyRelationEntity.class);
				Root<CourseSpeciatlyRelationEntity> subRoot = subquery
						.from(CourseSpeciatlyRelationEntity.class);
				subquery.select(subRoot.get("specialtyId"));
				Predicate p1 = cb.equal(subRoot.get("courseId"), courseId);
				Predicate p2 = cb.equal(subRoot.get("specialtyId"), root.get("id"));
				subquery.where(cb.and(p1, p2));
				predicates.add(cb.exists(subquery));
			}

			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		PageRequest pageRequest = PageRequest.of(curPage, pageSize,
				new Sort(Direction.DESC, "updateTime", "id"));

		Page<SpecialtyEntity> page = specialtyRepo.findAll(specification, pageRequest);
		return page;
	}

	@ApiOperation(value = "查询专业")
	@GetMapping("query")
	public List<SpecialtyEntity> query(@RequestParam(required = false) String name,
			@RequestParam(required = false) String code,
			@RequestParam(required = false) Boolean enable,
			@RequestParam(required = false) Long courseId) {

		User accessUser = getAccessUser();
		Long rootOrgId = accessUser.getRootOrgId();

		Specification<SpecialtyEntity> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();

			predicates.add(cb.equal(root.get("rootOrgId"), rootOrgId));

			if (StringUtils.isNotBlank(name)) {
				predicates.add(cb.like(root.get("name"), toSqlSearchPattern(name)));
			}
			if (StringUtils.isNotBlank(code)) {
				predicates.add(cb.like(root.get("code"), toSqlSearchPattern(code)));
			}
			if (null != enable) {
				predicates.add(cb.equal(root.get("enable"), enable));
			}

			if (null != courseId) {
				Subquery<CourseSpeciatlyRelationEntity> subquery = query
						.subquery(CourseSpeciatlyRelationEntity.class);
				Root<CourseSpeciatlyRelationEntity> subRoot = subquery
						.from(CourseSpeciatlyRelationEntity.class);
				subquery.select(subRoot.get("specialtyId"));
				Predicate p1 = cb.equal(subRoot.get("courseId"), courseId);
				Predicate p2 = cb.equal(subRoot.get("specialtyId"), root.get("id"));
				subquery.where(cb.and(p1, p2));
				predicates.add(cb.exists(subquery));
			}

			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		// 过载保护
		long total = specialtyRepo.count(specification);
		if (total > 1000) {
			List<SpecialtyEntity> list = Lists.newArrayList();
			return list;
		}

		List<SpecialtyEntity> list = specialtyRepo.findAll(specification,
				new Sort(Direction.DESC, "updateTime"));
		return list;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param orgId
	 * @param code
	 * @return
	 */
	@ApiOperation(value = "查询专业")
	@GetMapping("byCode")
	public SpecialtyEntity getByCode(@RequestParam Long orgId, @RequestParam String code) {
		SpecialtyEntity specialty = specialtyRepo.findByRootOrgIdAndCode(orgId, code);
		if (null == specialty) {
			throw new StatusException("620002", "专业不存在");
		}
		validateRootOrgIsolation(specialty.getRootOrgId());
		return specialty;
	}

	/**
	 * 修正
	 *
	 * @author WANGWEI
	 * @param domain
	 * @return
	 */
	@ApiOperation(value = "新增专业", notes = "新增")
	@PostMapping
	@Transactional
	public Long addSpecialty(@RequestBody SpecialtyDomain domain) {
		trim(domain, true);

		User accessUser = getAccessUser();
		Long rootOrgId = accessUser.getRootOrgId();

		String code = domain.getCode();
		if (StringUtils.isBlank(code)) {
			throw new StatusException("620001", "code is blank");
		}
		SpecialtyEntity course = specialtyRepo.findByRootOrgIdAndCode(rootOrgId, code);
		if (null != course) {
			throw new StatusException("620002", "专业编码已被占用");
		}

		SpecialtyInfo info = new SpecialtyInfo();
		info.setRootOrgId(rootOrgId);
		info.setCode(domain.getCode());
		info.setEnable(domain.getEnable());
		info.setId(domain.getId());
		info.setName(domain.getName());
		info.setEnable(domain.getEnable());

		SpecialtyEntity saved = specialtyService.saveSpecialty(info);
		return saved.getId();
	}

	/**
	 * 修正
	 *
	 * @author WANGWEI
	 * @param domain
	 * @return
	 */
	@ApiOperation(value = "修改专业", notes = "新增")
	@PutMapping
	@Transactional
	public Long updateSpecialty(@RequestBody SpecialtyDomain domain) {
		trim(domain, true);

		User accessUser = getAccessUser();
		Long rootOrgId = accessUser.getRootOrgId();

		SpecialtyInfo info = new SpecialtyInfo();
		info.setId(info.getId());
		info.setRootOrgId(rootOrgId);
		info.setCode(domain.getCode());
		info.setEnable(domain.getEnable());
		info.setId(domain.getId());
		info.setName(domain.getName());
		info.setEnable(domain.getEnable());

		SpecialtyEntity saved = specialtyService.saveSpecialty(info);
		return saved.getId();
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param ids
	 */
	@ApiOperation(value = "删除专业")
	@DeleteMapping("{ids}")
	@Transactional
	public void delete(@PathVariable String ids) {
		List<Long> specialtyIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		for (Long specialtyId : specialtyIds) {
			SpecialtyEntity one = GlobalHelper.getEntity(specialtyRepo, specialtyId,
					SpecialtyEntity.class);
			if (null == one) {
				continue;
			}
			validateRootOrgIsolation(one.getRootOrgId());
			specialtyRepo.delete(one);

			courseSpeciatlyRelationRepo.deleteBySpecialtyId(specialtyId);
		}
	}

	@ApiOperation(value = "下载导入模板", notes = "下载导入模板")
	@GetMapping("importTemplate")
	public void getDownloadTemplate(HttpServletResponse response) {
		String resoucePath = PathUtil.getResoucePath("templates/specialtyImportTemplate.xlsx");
		exportFile("专业导入模板.xlsx", new File(resoucePath));
	}

	@ApiOperation(value = "导入", notes = "导入")
	@PostMapping("import")
	@Transactional
	public Map<String, Object> importSpecialty(@RequestParam CommonsMultipartFile file) {
		DiskFileItem item = (DiskFileItem) file.getFileItem();
		File storeLocation = item.getStoreLocation();
		List<Map<String, Object>> failRecords = specialtyService.importSpecialty(getRootOrgId(),
				storeLocation);
		Map<String, Object> map = Maps.newHashMap();
		map.put("hasError", CollectionUtils.isNotEmpty(failRecords));
		map.put("failRecords", failRecords);
		return map;
	}

	@ApiOperation(value = "导出专业")
	@GetMapping("export")
	public void export(@RequestParam(required = false) String name,
			@RequestParam(required = false) String code,
			@RequestParam(required = false) Boolean enable,
			@RequestParam(required = false) Long courseId) {
		User accessUser = getAccessUser();

		Specification<SpecialtyEntity> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();

			predicates.add(cb.equal(root.get("rootOrgId"), accessUser.getRootOrgId()));

			if (StringUtils.isNotBlank(name)) {
				predicates.add(cb.like(root.get("name"), toSqlSearchPattern(name)));
			}
			if (StringUtils.isNotBlank(code)) {
				predicates.add(cb.like(root.get("code"), toSqlSearchPattern(code)));
			}
			if (null != enable) {
				predicates.add(cb.equal(root.get("enable"), enable));
			}

			if (null != courseId) {
				Subquery<CourseSpeciatlyRelationEntity> subquery = query
						.subquery(CourseSpeciatlyRelationEntity.class);
				Root<CourseSpeciatlyRelationEntity> subRoot = subquery
						.from(CourseSpeciatlyRelationEntity.class);
				subquery.select(subRoot.get("specialtyId"));
				Predicate p1 = cb.equal(subRoot.get("courseId"), courseId);
				Predicate p2 = cb.equal(subRoot.get("specialtyId"), root.get("id"));
				subquery.where(cb.and(p1, p2));
				predicates.add(cb.exists(subquery));
			}

			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		long count = specialtyRepo.count(specification);
		if (100000 < count) {
			throw new StatusException("620200", "数据量过大,无法导出");
		}

		List<SpecialtyEntity> list = specialtyRepo.findAll(specification);

		List<Object[]> datas = Lists.newArrayList();

		for (SpecialtyEntity cur : list) {
			datas.add(new Object[]{cur.getName(), cur.getCode()});
		}

		String filePath = systemConfig.getTempDataDir() + File.separator
				+ System.currentTimeMillis() + ".xlsx";
		File file = new File(filePath);

		ExcelWriter.write(EXCEL_HEADER, new Class[]{String.class, String.class}, datas,
				new File(filePath));

		exportFile("专业列表-" + getRootOrgId() + ".xlsx", file);

		FileUtils.deleteQuietly(file);
	}
}
