package cn.com.qmth.examcloud.core.basic.api.provider;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.persistence.criteria.Predicate;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.basic.api.OrgCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.OrgBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetOrgReq;
import cn.com.qmth.examcloud.core.basic.api.request.GetOrgsByIdListReq;
import cn.com.qmth.examcloud.core.basic.api.request.GetOrgsReq;
import cn.com.qmth.examcloud.core.basic.api.request.SaveOrgReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetOrgResp;
import cn.com.qmth.examcloud.core.basic.api.response.GetOrgsByIdListResp;
import cn.com.qmth.examcloud.core.basic.api.response.GetOrgsResp;
import cn.com.qmth.examcloud.core.basic.api.response.SaveOrgResp;
import cn.com.qmth.examcloud.core.basic.dao.OrgPropertyRepo;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.OrgEntity;
import cn.com.qmth.examcloud.core.basic.service.bean.OrgInfo;
import cn.com.qmth.examcloud.core.basic.service.impl.OrgServiceImpl;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.ApiOperation;

/**
 * {@link StatusException} 状态码范围：010XXX<br>
 *
 * @author WANGWEI
 * @date 2018年8月23日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@RestController
@RequestMapping("${$rmp.cloud.basic}" + "org")
public class OrgCloudServiceProvider extends ControllerSupport implements OrgCloudService {

	private static final long serialVersionUID = -7858439296389761341L;

	@Autowired
	private OrgServiceImpl orgService;

	@Autowired
	private OrgRepo orgRepo;

	@Autowired
	OrgPropertyRepo orgPropertyRepo;

	@ApiOperation(value = "保存机构")
	@PostMapping("saveOrg")
	@Transactional
	@Override
	public SaveOrgResp saveOrg(@RequestBody SaveOrgReq req) {
		OrgInfo info = new OrgInfo();
		info.setCode(req.getCode());
		info.setName(req.getName());
		info.setContacts(req.getContacts());
		info.setTelephone(req.getTelephone());
		info.setDomainName(req.getDomainName());
		info.setRemark(req.getRemark());

		OrgEntity saved = orgService.saveRootOrg(info);

		SaveOrgResp orgResp = new SaveOrgResp();
		orgResp.setId(saved.getId());
		orgResp.setName(saved.getName());
		return orgResp;
	}

	@ApiOperation(value = "获取机构")
	@PostMapping("getOrg")
	@Override
	public GetOrgResp getOrg(@RequestBody GetOrgReq req) {
		Long orgId = req.getOrgId();
		String orgCode = req.getOrgCode();
		Long rootOrgId = req.getRootOrgId();

		OrgEntity org = null;
		if (null != orgId) {
			org = GlobalHelper.getEntity(orgRepo, orgId, OrgEntity.class);
		} else if (StringUtils.isNotBlank(orgCode)) {
			if (null == rootOrgId) {
				throw new StatusException("150001", "rootOrgId is null");
			}
			org = orgRepo.findByRootIdAndCode(rootOrgId, orgCode);
		} else {
			throw new StatusException("150001", "orgId,orgCode不能都不为空");
		}

		if (null == org) {
			throw new StatusException("150001", "机构不存在");
		}

		OrgBean orgBean = new OrgBean();
		orgBean.setId(org.getId());
		orgBean.setName(org.getName());
		orgBean.setCode(org.getCode());
		orgBean.setParentId(org.getParentId());
		orgBean.setRootId(org.getRootId());
		orgBean.setEnable(org.getEnable());

		GetOrgResp resp = new GetOrgResp();
		resp.setOrg(orgBean);
		return resp;
	}

	@ApiOperation(value = "按ID集合查询机构")
	@PostMapping("getOrgsByIdList")
	@Override
	public GetOrgsByIdListResp getOrgsByIdList(@RequestBody GetOrgsByIdListReq req) {

		List<Long> orgIdList = req.getOrgIdList();

		final List<OrgBean> orgList = Lists.newArrayList();
		for (Long cur : orgIdList) {
			OrgEntity org = GlobalHelper.getEntity(orgRepo, cur, OrgEntity.class);
			if (null == org) {
				throw new StatusException("010001", "no org. [id=" + cur + "]");
			}
			OrgBean orgBean = new OrgBean();
			orgBean.setId(org.getId());
			orgBean.setName(org.getName());
			orgBean.setCode(org.getCode());
			orgBean.setParentId(org.getParentId());
			orgBean.setRootId(org.getRootId());
			orgBean.setEnable(org.getEnable());

			orgList.add(orgBean);
		}
		GetOrgsByIdListResp resp = new GetOrgsByIdListResp();
		resp.setOrgList(orgList);
		return resp;
	}

	@ApiOperation(value = "查询所有机构")
	@PostMapping("getOrgs")
	@Override
	public GetOrgsResp getOrgs(@RequestBody GetOrgsReq req) {
		Boolean withoutParentId = req.getWithoutParentId();
		Long rootOrgId = req.getRootOrgId();
		Boolean enable = req.getEnable();

		final long start = null == req.getStart() ? 1 : req.getStart();

		Pageable pageable = PageRequest.of(0, 100, Sort.Direction.ASC, "id");

		Specification<OrgEntity> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();

			if (null != rootOrgId) {
				predicates.add(cb.equal(root.get("rootId"), rootOrgId));
			}

			predicates.add(cb.greaterThanOrEqualTo(root.get("id"), start));

			if (null != withoutParentId) {
				if (withoutParentId) {
					predicates.add(cb.isNull(root.get("parentId")));
				} else {
					predicates.add(cb.isNotNull(root.get("parentId")));
				}
			}

			if (null != enable) {
				predicates.add(cb.equal(root.get("enable"), enable));
			}

			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		Page<OrgEntity> page = orgRepo.findAll(specification, pageable);

		Iterator<OrgEntity> iterator = page.iterator();

		List<OrgBean> list = Lists.newArrayList();
		long next = start;
		boolean has = false;
		while (iterator.hasNext()) {
			OrgEntity org = iterator.next();
			OrgBean orgBean = new OrgBean();
			orgBean.setId(org.getId());
			orgBean.setName(org.getName());
			orgBean.setCode(org.getCode());
			orgBean.setParentId(org.getParentId());
			orgBean.setRootId(org.getRootId());
			orgBean.setEnable(org.getEnable());

			next = org.getId();
			list.add(orgBean);
			has = true;
		}

		GetOrgsResp resp = new GetOrgsResp();
		if (has) {
			next++;
		}
		resp.setNext(next);
		resp.setOrgBeanList(list);

		return resp;
	}

}
