package cn.com.qmth.examcloud.core.basic.base.util;

import cn.com.qmth.examcloud.commons.exception.StatusException;

import javax.imageio.ImageIO;
import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.Serializable;
import java.util.Random;

/**
 * 数字公式验证码
 */
public class VerifyCode {

    private static final char[] OPS = new char[]{'+', '-'};

    /**
     * 生成验证码图片
     */
    public static Result generateVerifyCode() {
        int width = 100;
        int height = 40;

        BufferedImage image = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);

        Graphics graphics = image.getGraphics();// 画笔
        graphics.setColor(new Color(0xDCDCDC));// 背景色
        graphics.fillRect(0, 0, width, height);

        graphics.setColor(Color.black);// 边框色
        graphics.drawRect(0, 0, width - 1, height - 1);

        // 生成N个干扰点
        Random random = new Random();
        for (int i = 0; i < 500; i++) {
            int x = random.nextInt(width);
            int y = random.nextInt(height);
            graphics.drawOval(x, y, random.nextInt(2), random.nextInt(2));
        }

        // 生成N条干扰线
        for (int i = 0; i < 3; i++) {
            int x1 = random.nextInt(width);
            int y1 = random.nextInt(height);
            int x2 = random.nextInt(width);
            int y2 = random.nextInt(height);
            graphics.drawLine(x1, y1, x2, y2);
        }

        // 随机生成三个数字、并组合成一个数学公式
        String verifyCode = new StringBuilder()
                .append(random.nextInt(10))
                .append(OPS[random.nextInt(2)])
                .append(random.nextInt(10))
                .append(OPS[random.nextInt(2)])
                .append(random.nextInt(10))
                .toString();

        graphics.setColor(new Color(0, 0, 255));
        graphics.setFont(new Font("宋体", Font.BOLD, 22));
        graphics.drawString(verifyCode, 12, 28);
        graphics.dispose();

        Integer result;
        try {
            // 将验证码的字符串算出结果
            ScriptEngineManager manager = new ScriptEngineManager();
            ScriptEngine engine = manager.getEngineByName("JavaScript");
            result = (Integer) engine.eval(verifyCode);
        } catch (Exception e) {
            throw new StatusException("500", "验证码图片生成失败！", e);
        }

        return new Result(image, result);
    }

    public static byte[] imageToBytes(BufferedImage image) {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        try {
            ImageIO.write(image, "JPEG", out);
        } catch (IOException e) {
            throw new StatusException("500", "验证码图片处理失败！", e);
        }
        return out.toByteArray();
    }

    public static class Result implements Serializable {

        private static final long serialVersionUID = 1L;

        private BufferedImage image;// 验证码图片

        private Integer result;// 验证码结果

        public Result(BufferedImage image, Integer result) {
            this.image = image;
            this.result = result;
        }

        public Result() {

        }

        public BufferedImage getImage() {
            return image;
        }

        public void setImage(BufferedImage image) {
            this.image = image;
        }

        public Integer getResult() {
            return result;
        }

        public void setResult(Integer result) {
            this.result = result;
        }
    }

    /*public static void main(String[] args) throws Exception {
        final String dir = "/home/verifyCodes";
        new File(dir).mkdirs();

        for (int n = 1; n <= 10; n++) {
            Result result = VerifyCode.generateVerifyCode();

            final String imgPath = String.format("Img_%s.jpg", n);
            File imgFile = new File(dir + "/" + imgPath);
            ImageIO.write(result.getImage(), "JPEG", imgFile);
            System.out.println(imgPath + " \t\t " + result.getResult());

            byte[] bytes = imageToBytes(result.getImage());
            String base64 = Base64Utils.encodeToString(bytes);
            // System.out.println(base64 + "\n");
        }
    }*/

}
